#include <cstddef>
#include <iostream>

using std::size_t;


void copy_array(const double* from, double* to, size_t size) {
    for (size_t i = 0; i < size; ++i) to[i] = from[i];
}

void resize_array(double*& arr, size_t oldSize, size_t newSize) {
    double* newArr = new double[newSize];
    size_t minSize = (oldSize < newSize) ? oldSize : newSize;
    copy_array(arr, newArr, minSize);
    delete[] arr;
    arr = newArr;
}


class vector {
public:
    ~vector();
    vector();
    vector(size_t sz, double val);

    void push_back(double val);
    size_t size() const;
    size_t capacity() const;
    double at(size_t i) const;
    void reserve(size_t cap);

private:
    double* m_data;
    size_t m_capacity;
    size_t m_size;
};

vector::~vector() {
    delete[] m_data;
}

vector::vector() :
    m_data(nullptr),
    m_capacity(0),
    m_size(0)
{}

vector::vector(size_t sz, double val) :
    m_data(new double[sz]),
    m_capacity(sz),
    m_size(sz)
{
    for (size_t i = 0; i < sz; ++i) m_data[i] = val;
}

void vector::reserve(size_t cap) {
    if (m_capacity < cap) {
        resize_array(m_data, m_size, cap);
        m_capacity = cap;
    }
}

void vector::push_back(double val) {
    if (m_size == m_capacity) {
        reserve((m_capacity + 1) * 2);
    }

    m_data[m_size] = val;
    m_size++;
}

size_t vector::size() const {
    return m_size;
}

size_t vector::capacity() const {
    return m_capacity;
}

double vector::at(size_t i) const {
    return m_data[i];
}

void print_vector(const vector& v) {
    std::cout << "capacity: " << v.capacity() << " size: " << v.size() << " data: ";
    
    for (size_t i = 0; i < v.size(); i++) {
        std::cout << v.at(i) << ' ';
    }
    std::cout << '\n';
}


int main() {
    vector v(2, 4.56);
    v.push_back(1.23);
    print_vector(v);            // 1.23 2.34 3.45
    vector v2 = v;
}
